"""
This script carries out a ground state calculations in order
to generate wave functions for different configurations.
"""
from ase.io import read
from gpaw import GPAW, PW
import numpy as np
import argparse


def get_number_of_bands(atoms, factor=2.0):
    """
    Get the number of bands that should be converged.

    Parameters
    ----------
    atoms : ASE Atoms

    Returns
    -------
    int
        Number of bands
    """
    electrons = {'Au': 11, 'Cu': 11, 'Ag': 11,
                 'Pd': 10, 'Pt': 10}
    nelectrons = 0
    for sym in atoms.get_chemical_symbols():
        nelectrons += electrons[sym]

    return int(factor * nelectrons / 2)


def perform_gs_calculation(atoms, tag, nk):
    """
    Perform a ground state calculation with GPAW.

    Parameters
    ----------
    atoms : ASE Atoms
        atomic structure
    tag : str
        Arbitrary string used for naming files
    nk : int
        Number of k points in one direction in primitive cell
    """
    # Calculation parameters
    alat = (4 * atoms.get_volume() / len(atoms)) ** (1 / 3.)
    kdens = nk * alat / (2.0 * np.pi) / np.sqrt(2) / 1.25

    if len(atoms) > 1:
        nbands = get_number_of_bands(atoms)
        nbands_extra = 10
    else:
        nbands = 12
        nbands_extra = 4

    # Filenames
    stdout_file = 'gscalc-' + tag + '.out'
    outfile = 'gs-' + tag + '.gpw'

    # Calculate ground state
    calc = GPAW(mode=PW(),
                kpts={'density': kdens,
                      'even': False,
                      'gamma': True},
                nbands=nbands + nbands_extra,
                xc='GLLBSCM',
                setups={'Ag': '11', 'Pd': '10', 'Pt': '10'},
                convergence={'energy': 1e-5,
                             'bands': nbands},
                txt=stdout_file)
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write(outfile, 'all')


parser = argparse.ArgumentParser()
parser.add_argument('structure', type=str,
                    help='Input structure file')
parser.add_argument('tag', type=str,
                    help='Arbitrary tag to identify structure')
parser.add_argument('nk', type=int,
                    help='The density of k-points will be equivalent to ' +
                    'nk x nk x nk k-points in the primitive reciprocal cell.')
args = parser.parse_args()

atoms = read(args.structure)
perform_gs_calculation(atoms, args.tag, args.nk)
