import os
import pandas as pd
import numpy as np
from scipy.interpolate import interp1d


def redlich_kister(x, *coeffs):
    y = np.zeros(len(x))
    for p in range(len(coeffs)):
        y += coeffs[p] * (1 - 2 * x)**p
    y *= x * (1 - x)
    return y


class DielectricFunction():

    def __init__(self, element1, element2, concentration):
        """
        An alloy based on any binary combination of Ag, Au, Cu, Pd and Pt.

        Parameters
        ----------
        element1 : string
            Chemical element such as `'Ag'`
        element2 : string
            Chemical element such as `'Au'`
        concentration : float
            Concentration of `element2` as a fraction (`0.5` means 50 %)
        """
        available_elements = ['Ag', 'Au', 'Cu', 'Pd', 'Pt']
        if element1 not in available_elements or \
           element2 not in available_elements:
            raise Exception('Binary alloy {}-{} is not available.')
        if element1 == element2:
            raise Exception('Please specify two different elements.')

        self.element1 = element1
        self.element2 = element2

        tol = 1e-5
        if concentration < -tol or concentration > 1 + tol:
            raise ValueError('Concentration must lie in the range 0 <= c <= 1')
        self.concentration = concentration

        # Read data and single out the relevant part
        script_dir = os.path.dirname(__file__)
        rel_path = '../auxiliary_data'
        filename = os.path.join(script_dir, rel_path)
        filename = os.path.join(filename, 'expansion-coefficients.data')

        data = pd.read_csv(filename, delim_whitespace=True, header=None,
                           skiprows=7, usecols=list(range(11)))
        order = 4
        data.columns = ['element1', 'element2', 'energy', 'type'] + \
            ['coeff_{}'.format(i) for i in range(order + 1)] + \
            ['eps0', 'eps1']

        data = data.loc[data.element1 == min([self.element1, self.element2])]
        data = data.loc[data.element2 == max([self.element1, self.element2])]
        data = data.sort_values(by='energy')

        # Save the dielectric function of both real and imaginary part as
        # a function of energy (at the specified concentration)
        self.real = []
        self.imag = []
        self.energies_real = []
        self.energies_imag = []
        for _, row in data.iterrows():
            coeffs = [row[f'coeff_{i}'] for i in range(order + 1)]
            if self.element2 > self.element1:
                c = self.concentration
            else:
                c = 1 - self.concentration
            eps0 = row.eps0
            eps1 = row.eps1
            eps = (redlich_kister(np.array([c]), *coeffs)[0]
                   + c * eps1 + (1 - c) * eps0)
            if row.type == 'real':
                self.real.append(eps)
                self.energies_real.append(row.energy)
            elif row.type == 'imag':
                self.imag.append(eps)
                self.energies_imag.append(row.energy)
            else:
                raise Exception(
                    'Unknown type of dielectric function {}'.format(row.type))

        # Interpolate data such that the dielectric function
        # can be extracted at arbitrary energies (within available interval)
        self.dielectric_function_real = interp1d(self.energies_real, self.real,
                                                 kind='slinear',
                                                 bounds_error=True)
        self.dielectric_function_imag = interp1d(self.energies_imag, self.imag,
                                                 kind='slinear',
                                                 bounds_error=True)

    def __str__(self):
        return 'Binary alloy {}-{} with {:.2f} % {}'.format(
            self.element1, self.element2,
            100 * self.concentration, self.element2)

    def get_dielectric_function(self, energy):
        """
        Return dielectric function at specified energy.

        Parameters
        ----------
        energy : float or NumPy array with floats
            Energy(ies) in eV

        Returns
        -------
        complex
            Dielectric function at specified energy
        """
        return self.dielectric_function_real(energy) + \
            1j * self.dielectric_function_imag(energy)


if __name__ == '__main__':

    import matplotlib.pyplot as plt
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('element1', type=str,
                        help='Element 1, such as Au')
    parser.add_argument('element2', type=str,
                        help='Element 2, such as Pd')
    parser.add_argument('concentration', type=float,
                        help='Concentration of element2, such as 0.75')
    args = parser.parse_args()

    df = DielectricFunction(args.element1, args.element2, args.concentration)

    fix, ax = plt.subplots()
    energies = np.linspace(0, 12, 121)
    ax.plot(energies, df.get_dielectric_function(energies).imag,
            label='{}{:.3f}-{}{:.3f}'.format(args.element1,
                                             1 - args.concentration,
                                             args.element2,
                                             args.concentration))
    ax.legend()
    ax.set_xlim([0, 12])
    ax.set_ylim([0, 12])

    ax.set_xlabel('Energy (eV)')
    ax.set_ylabel('Imaginary part of dielectric function')
    plt.show()
